(in-package #:trivial-adjust-simple-array/tests)


(define-test grow-size
  :parent trivial-adjust-simple-array
  (let* ((array (make-array 1 :element-type 'fixnum))
         (new-size 2)
         (new-array (adjust-simple-array array new-size :element-type 'fixnum)))
    (is = new-size (length new-array))))

(define-test shrink-size
  :parent trivial-adjust-simple-array
  (let* ((array (make-array 2 :element-type 'fixnum))
         (new-size 1)
         (new-array (adjust-simple-array array new-size :element-type 'fixnum)))
    (is = new-size (length new-array))))

(define-test still-simple
  :parent trivial-adjust-simple-array
  (let* ((array (make-array 1 :element-type 'fixnum))
         (new-array (adjust-simple-array array 2 :element-type 'fixnum)))
    (false (array-displacement       new-array))
    (false (array-has-fill-pointer-p new-array))
    (false (adjustable-array-p       new-array))))

(define-test array-type
  :parent trivial-adjust-simple-array
  (is typep 'simple-bit-vector
      (adjust-simple-array
       (make-array 1 :element-type 'bit)
       2 :element-type 'bit :initial-element 0))
  (is typep 'simple-array
      (adjust-simple-array
       (make-array 1 :element-type 'fixnum)
       2 :element-type 'fixnum :initial-element 0))
  (is typep 'simple-array
      (adjust-simple-array
       (make-array 1 :element-type 'list)
       2 :element-type 'list))
  (is typep 'simple-vector
      (adjust-simple-array
       (make-array 1)
       2 :element-type 't)))

(define-test type
  :parent trivial-adjust-simple-array
  #-abcl ;; https://github.com/armedbear/abcl/issues/673
  (let* ((array (make-array 1 :element-type '(unsigned-byte 32)))
         (new-array (adjust-simple-array array 2
                                         :element-type '(unsigned-byte 32))))
    (is equalp #-(or ecl clasp mkcl) '(unsigned-byte 32)
        #+(or ecl clasp) 'ext:byte32
        #+mkcl 'mkcl:natural32
        (array-element-type new-array)))
  (let* ((array (make-array 1 :element-type 'single-float))
         (new-array (adjust-simple-array array 2 :element-type 'single-float)))
    (is eq #-(or abcl clisp) 'single-float
        #+(or abcl clisp) t
        (array-element-type new-array))))

(define-test type-mismatch
  :parent trivial-adjust-simple-array
  (fail
   (let* ((array (make-array 1 :element-type 'fixnum))
          (new-array
            (adjust-simple-array array 2 :element-type 'character))))))

(define-test contents
  :parent trivial-adjust-simple-array
  (let* ((array (make-array 3 :element-type 'fixnum :initial-contents '(1 2 3)))
         (new-array (adjust-simple-array array 5
                                         :element-type 'fixnum
                                         :initial-element 42)))
    (is equalp #(1 2 3 42 42) new-array)))

(define-test modfiy-macro
  :parent trivial-adjust-simple-array
  (let ((array (make-array 1 :element-type 'fixnum))
        (new-size 2))
    (adjust-simple-arrayf array new-size
                          :element-type 'fixnum
                          :initial-element 0)
    (is = new-size (length array))
    (false (array-displacement       array))
    (false (array-has-fill-pointer-p array))
    (false (adjustable-array-p       array))))

(define-test nil-initial-element
  :parent trivial-adjust-simple-array
  (let ((array (make-array 0 :element-type '(or null fixnum)))
        (new-size 1))
    (adjust-simple-arrayf array new-size
                          :element-type '(or null fixnum)
                          :initial-element nil)
    (is eq nil (aref array 0))))
