(defpackage trivial-adjust-simple-array
  (:use :cl)
  (:export
   #:adjust-simple-array
   #:adjust-simple-arrayf))

(in-package :trivial-adjust-simple-array)


(defmacro adjust-simple-array (array size
                               &key
                               element-type
                               (initial-element nil initial-element-supplied-p))
  "Changes the size of simple array. The result is simple array of the same type
as array, that has the given new SIZE, ELEMENT-TYPE and (optionally)
INITIAL-ELEMENT in a new uninitialized elements."
  (let ((initial-clause (when initial-element-supplied-p
                          `(:initial-element ,initial-element))))
    #-(or ecl mkcl clasp allegro)
    `(adjust-array ,array ,size :element-type ,element-type ,@initial-clause)
    #+(or ecl mkcl clasp allegro)
    (let ((result (gensym "RESULT"))
          #+allegro (old-type (gensym "OLD-TYPE")))
      `(progn
         ;; NOTE: allegro does not bother to do type checking in REPLACE
         #+allegro (let ((,old-type (second (type-of ,array))))
                     (unless (equalp (upgraded-array-element-type ,element-type)
                                     ,old-type)
                       (error "Element type mismatch: was ~a, requested ~a"
                              ,old-type ,element-type)))
         (let ((,result (make-array ,size
                                    :element-type ,element-type
                                    ,@initial-clause)))
           (replace ,result ,array)
           ,result)))))

#+(or ecl mkcl clasp clisp) (declaim (inline adjust-simple-array*))
#+(or ecl mkcl clasp clisp)
(defun adjust-simple-array* (array size &key element-type initial-element)
  "HACK: these lisps require an actual function in DEFINE-MODIFY-MACRO."
  (adjust-simple-array array size
                       :element-type element-type
                       :initial-element initial-element))

(define-modify-macro adjust-simple-arrayf
    (size &rest keywords)
  #-(or ecl mkcl clasp clisp) adjust-simple-array
  #+(or ecl mkcl clasp clisp) adjust-simple-array*
  "SETF-like macro to change simple array size in-place.")
